import {
  IEventParam, Loader, Metadata, IDictionary, I18n, Http, Datasource,
} from '@zeedhi/core';

import {
  GridEditable, Card, Modal, ModalService, Button, Form, TextInput, DialogService,
} from '@zeedhi/common';
  
export class Ger01800UnidadeMedidaController {
  private conversaoIsVisible = false;
  
  public gridEditController = Loader.getInstance('GridEditableController');

  public crudController = Loader.getInstance('CrudController');
  
  public modalController = Loader.getInstance('ModalController');

  public utilities = Loader.getInstance('Utilities');

  private modalConversao!: Modal;

  public onChangeUnMedidaCurrentRow({ component }: IEventParam<Datasource>) {
    const rowColumn = component.currentRow.SGUNIDADE;

    if (rowColumn) {
      const gridConversao = Metadata.getInstance<GridEditable>('conversaoGridDetail');
  
      const conditions = [
        {
          name: 'SGUNIDADE',
          operator: '=',
          value: rowColumn,
        },
      ];

      gridConversao.datasource.filter = conditions;
      // this.gridEditController.reload({ component: gridConversao });
      gridConversao.datasource.get();
      this.cardUnMedidaVisible();
    }
  }

  setRelacao({ row }: IDictionary) {
    return `${row.VRFTCONVER} ${row.DSUNIDPARA} (s) ${I18n.translate('pme::equivaleA')} 1 ${row.DSUNIDDE}`;
  }

  private cardUnMedidaVisible() {
    const card = Metadata.getInstance<Card>('conversaoCard');
    try {
      if (!this.gridEditController.newRow) {
        card.isVisible = true;
        this.conversaoIsVisible = true;
      }
    } catch (Error) {
      card.isVisible = false;
      this.conversaoIsVisible = false;
    }
  }
  
  public closeCardUnMedida() {
    const card = Metadata.getInstance<Card>('conversaoCard');
    
    card.isVisible = false;
    this.conversaoIsVisible = false;
  }
  
  public createModal() {
    this.modalController.createModal('/modals/formTpCardapio');
  }
  
  get gridColsReturnValue(): number {
    return this.conversaoIsVisible ? 6 : 12;
  }

  public saveUnMedida() {
    const gridUnMedida = Metadata.getInstance<GridEditable>('gridUnidadeMedida');
    const editedRows: IDictionary = gridUnMedida.getEditedRows();

    if (editedRows.length > 0) {
      this.crudController.saveRows(editedRows, 'unidMedida', gridUnMedida).then(() => {
        this.gridEditController.defaultState(gridUnMedida);
        gridUnMedida.datasource.get();
        this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
        this.closeCardUnMedida();
      }).catch((error: string) => {
        this.utilities.showAlert('alert', `pme::${error}`, 'error');
      });
    }
  }
  
  public reloadUnMedida() {
    const gridConversao = Metadata.getInstance<GridEditable>('gridUnidadeMedida');
    this.gridEditController.reload({ component: gridConversao });
    this.closeCardUnMedida();
  }

  public cancelEditing() {
    const gridConversao = Metadata.getInstance<GridEditable>('gridUnidadeMedida');
    this.gridEditController.cancelEditing({ component: gridConversao });
    this.closeCardUnMedida();
  }

  public addConversao() {
    if (this.utilities.centralIsValid()) {
      if (!this.modalConversao) {
        this.createModalAddConversao().then(() => {
          this.modalConversao.show();
        });
      } else {
        this.modalConversao.show();
      }
    }
  }

  private async createModalAddConversao() {
    this.modalConversao = await ModalService.createFromJson('modals/ger01800_addConversao', true);
  }

  public closeConversaoModalAdd() {
    this.modalConversao.hide();
  }
  
  public setfullscreen({ component }: IEventParam<Button>) {
    this.modalConversao.fullscreen = !this.modalConversao.fullscreen;
    this.modalConversao.cssClass = this.modalConversao.fullscreen ? 'fullscreen-fill-content' : 'modal-emissao-cardapio';
    
    component.iconName = (this.modalConversao.fullscreen) ? 'mdi-fullscreen-exit' : 'mdi-fullscreen';
  }

  private setParams(rowConversao: IDictionary) {
    rowConversao.DSUNIDADE = rowConversao.SGUNIDPARA.DSUNIDADE;
    rowConversao.SGUNIDADE = rowConversao.SGUNIDPARA.SGUNIDADE;
  }

  public setOpcRelacao() {
    const gridUnidMedida = Metadata.getInstance<GridEditable>('gridUnidadeMedida');
    const dsUnidade = gridUnidMedida.datasource.currentRow.DSUNIDADE;
    const formConversao = Metadata.getInstance<Form>('formConversao');
    const rowConversao = formConversao.value;
    const opcRelacao = formConversao.children.find((name) => name.name === 'OPCRELACAO');
    const valorX = Metadata.getInstance<TextInput>('VRFTCONVER').value;

    this.setParams(rowConversao);

    if (formConversao.value.SGUNIDPARA && opcRelacao && valorX) {
      opcRelacao.datasource.data[0].name = `1 ${dsUnidade} ${I18n.translate('pme::equivaleA')} ${valorX} ${rowConversao.DSUNIDADE}(s)`;
      opcRelacao.datasource.data[1].name = `1 ${rowConversao.DSUNIDADE} ${I18n.translate('pme::equivaleA')} ${valorX} ${dsUnidade}(s)`;
    } else if (formConversao.value.SGUNIDPARA && opcRelacao) {
      opcRelacao.datasource.data[0].name = `1 ${dsUnidade} ${I18n.translate('pme::equivaleA')} X ${rowConversao.DSUNIDADE}(s)`;
      opcRelacao.datasource.data[1].name = `1 ${rowConversao.DSUNIDADE} ${I18n.translate('pme::equivaleA')} X ${dsUnidade}(s)`;
    }
  }

  public setValueUnPrinc() {
    const gridUnidMedida = Metadata.getInstance<GridEditable>('gridUnidadeMedida');
    const sgUnidade = gridUnidMedida.datasource.currentRow.SGUNIDADE;
    const dsUnidade = gridUnidMedida.datasource.currentRow.DSUNIDADE;
    const formConversao = Metadata.getInstance<Form>('formConversao');
    const opcRelacao = formConversao.children.find((name) => name.name === 'OPCRELACAO');
    const unidPrinc = Metadata.getInstance<TextInput>('SGUNIDDE');
    
    unidPrinc.value = `${sgUnidade} | ${dsUnidade}`;
    
    if (opcRelacao) {
      opcRelacao.datasource.data[0].name = `1 ${dsUnidade} ${I18n.translate('pme::equivaleA')} X(s)`;
      opcRelacao.datasource.data[1].name = `X ${I18n.translate('pme::equivaleA')} 1 ${dsUnidade}(s)`;
    }
  }

  public saveConversao({ component }: IEventParam<Button>) {
    const gridUnidMedida = Metadata.getInstance<GridEditable>('gridUnidadeMedida');
    const gridConversao = Metadata.getInstance<GridEditable>('conversaoGridDetail');
    const formConversao = Metadata.getInstance<Form>('formConversao');
    const row = formConversao.value;
    row.SGUNIDDE = gridUnidMedida.datasource.currentRow.SGUNIDADE;
    row.IDTIPOPE = '*';

    if (formConversao.validate()) {
      this.utilities.activeLoadingButton(component);
      Http.post('/unidadeMedida/save', { requestType: 'Row', row }).then((response) => {
        if (response.data.dataset.save[0]) {
          gridConversao.datasource.get();
          this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
          this.closeConversaoModalAdd();
        }
      }).catch((error) => {
        this.utilities.showAlert('alert', error, 'error');
      }).finally(() => {
        this.utilities.disableLoadingButton(component);
      });
    }
  }

  public confirmDeleteConversao({ component }: IDictionary) {
    const grid = this.utilities.returnGrid(component) as GridEditable;
    const checkedRows = grid.selectedRows;
    if (checkedRows.length === 0) this.utilities.showAlert('alert', 'pme::nenhumRegSelecionado', 'warning');
    else {
      DialogService.show({
        name: 'excluiConversao',
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::desejaExcluirRegistros',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.deleteConversao() },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    }
  }

  public deleteConversao() {
    const gridConversao = Metadata.getInstance<GridEditable>('conversaoGridDetail');
    const checkedRows = gridConversao.selectedRows;
    if (checkedRows.length) {
      Http.post('/unidadeMedida/delete', { requestType: 'Row', checkedRows }).then(() => {
        gridConversao.datasource.get();
        gridConversao.selectedRows = [];
        this.utilities.showAlert('alertSuccess', 'pme::excluidoComSucesso', 'success');
        DialogService.hide();
      });
    }
  }
}
