import {
  Metadata, IEventParam, Loader, IDictionary, Config, dayjs as moment,
} from '@zeedhi/core';

import {
  GridEditable, Modal, ModalService, Form, Button, DialogService, Select,
} from '@zeedhi/common';
import { PmeMask } from '@/config';

export class Ger04000NutrientesPorProdutoController {
  public gridEditableController = Loader.getInstance('GridEditableController');
  
  private utilities = Loader.getInstance('Utilities');
  
  public modalController = Loader.getInstance('ModalController');
  
  public reportController = Loader.getInstance('ReportController');
  
  public crudController = Loader.getInstance('CrudController');
  
  private modalAddNutri!: Modal;
  
  private modalRelatorioNutriProd!: Modal;
  
  public disablePropChange = false;
  
  public filterNutrientes() {
    const produto = Metadata.getInstance<Select>('CDPRODUTO').value;
    const nutriente: GridEditable = Metadata.getInstance('NUTRIENTE');
    const form = Metadata.getInstance<Form>('formFilter').value;
    if (form.CDPRODUTO !== null) {
      const conditions = [
        {
          name: 'CDPRODUTO',
          operator: '=',
          value: produto,
        },
      ];
      nutriente.datasource.filter = conditions;
      nutriente.datasource.get();
    } else nutriente.datasource.data = [];
  }
  
  public addNutriente() {
    const produto = Metadata.getInstance<Select>('CDPRODUTO').value;
    if (this.utilities.centralIsValid() && produto) {
      if (!this.modalAddNutri) {
        this.createModalAddNutrientes().then(() => {
          this.modalAddNutri.show();
          this.filterDataSourceAddNutri();
        });
      } else {
        this.modalAddNutri.show();
      }
    }
  }
  
  private async createModalAddNutrientes() {
    this.modalAddNutri = await ModalService.createFromJson('modals/ger04000_nutriente', true);
  }
  
  public closeNutriModalAdd() {
    const gridAddNutri = Metadata.getInstance<GridEditable>('gridAddNutriente');
    this.gridEditableController.cancelEditing({ component: gridAddNutri });
    this.modalAddNutri.hide();
  }
  
  public filterDataSourceAddNutri() {
    const gridAddNutri = Metadata.getInstance<GridEditable>('gridAddNutriente');
    const produto = Metadata.getInstance<Select>('CDPRODUTO').value;

    const conditions = [
      {
        name: 'CDPRODUTO',
        operator: '=',
        value: produto,
      },
    ];

    gridAddNutri.datasource.filter = conditions;
    gridAddNutri.datasource.get();
  }
  
  public saveAddNutrientes() {
    const gridGrupoNutri: GridEditable = Metadata.getInstance('gridAddNutriente');
    const editedRows: IDictionary = gridGrupoNutri.getEditedRows();
    
    if (gridGrupoNutri.selectedRows.length && editedRows.length === 0) this.utilities.showAlert('alertError', 'pme::qtdFonteVazio', 'warning');
    if (editedRows.length > 0) {
      const gridNutrientes: GridEditable = Metadata.getInstance('NUTRIENTE');
      const produto = Metadata.getInstance<Select>('CDPRODUTO').value;
      const buttonAdd = Metadata.getInstance<Button>('saveButton');
      const buttonCancel = Metadata.getInstance<Button>('cancelButton');
      let fonte;

      editedRows.forEach((row: IDictionary) => {
        row.CDPRODUTO = produto;
        row.DTULTATUNUPR = moment().format('DD/MM/YYYY HH:mm:ss');
        if (!row.QTNUTRPROD) row.QTNUTRPROD = 0;
        if (!row.DSFONTNUTR) fonte = '';
        row.__is_new = true;
      });
      
      if (fonte === '') this.utilities.showAlert('alertError', 'pme::qtdFonteVazio', 'warning');
      else {
        this.crudController.saveRows(editedRows, 'nutrprod', gridGrupoNutri).then(() => {
          gridNutrientes.datasource.get();
          buttonAdd.disabled = true;
          buttonCancel.disabled = true;
          this.gridEditableController.reload({ component: gridNutrientes });
          this.gridEditableController.reload({ component: gridGrupoNutri });
          this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
        }).catch((error: string) => {
          this.utilities.showAlert('alert', `pme::${error}`, 'error');
        });
      }
    }
  }
  
  public async deleteCheckedRows() {
    const gridNutrientes: GridEditable = Metadata.getInstance('NUTRIENTE');
    const checkedRows = gridNutrientes.selectedRows;
    
    if (checkedRows.length === 0) this.utilities.showAlert('alert', 'pme::nenhumRegSelecionado', 'warning');
    else {
      DialogService.show({
        name: 'confirmDeleteNutri',
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::desejaExcluirRegistros',
        buttons: [
          {
            name: 'yes',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.deleteNutriente() },
            outline: true,
          },
          {
            name: 'no',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
        ],
      });
    }
  }
  
  public deleteNutriente() {
    const gridNutrientes: GridEditable = Metadata.getInstance('NUTRIENTE');
    const checkedRows = gridNutrientes.selectedRows;
    const dataSourceName = '/nutrprod';
    this.crudController.deleteRows(checkedRows, dataSourceName).then(() => {
      gridNutrientes.cancelEditedRows();
      gridNutrientes.datasource.get();
      gridNutrientes.selectedRows = [];
      DialogService.hide();
      
      this.utilities.showAlert('alertError', 'pme::excluidoComSucesso', 'success');
    }).catch((error: string) => {
      this.utilities.showAlert('alert', `pme::${error}`, 'error');
      DialogService.hide();
    });
  }
  
  public saveNutrientes() {
    const nutriente: GridEditable = Metadata.getInstance('NUTRIENTE');
    const editedRows: IDictionary = nutriente.getEditedRows();
    if (editedRows.length > 0) {
      const produto = Metadata.getInstance<Select>('CDPRODUTO').value;
      
      editedRows.forEach((row: IDictionary) => {
        row.CDPRODUTO = produto;
        row.DTULTATUNUPR = moment().format('DD/MM/YYYY HH:mm:ss');
        if (!row.QTNUTRPROD) row.QTNUTRPROD = 0;
        row.__is_new = false;
      });

      this.crudController.saveRows(editedRows, 'nutrprod', nutriente).then(() => {
        nutriente.datasource.get();
        this.gridEditableController.reload({ component: nutriente });
        this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
      }).catch((error: string) => {
        this.utilities.showAlert('alert', `pme::${error}`, 'error');
      });
    }
  }
    
  public openModalRelatorioNutriProd() {
    if (!this.modalRelatorioNutriProd) {
      this.createModalRelatorioNutriProd().then(() => {
        this.modalRelatorioNutriProd.show();
      });
    } else {
      this.modalRelatorioNutriProd.show();
    }
  }
  
  private async createModalRelatorioNutriProd() {
    this.modalRelatorioNutriProd = await ModalService.createFromJson('modals/ger04000_relatorio_nutri_prod', true);
  }
  
  public cancelModalRelatorioNutriProd() {
    this.modalRelatorioNutriProd.hide();
  }
  
  private setParams(row: IDictionary) {
    const mask: string = (Config.masks.productMask as PmeMask).pattern;
    row.PRODUCTMASK = this.utilities.formatReportMask(mask);
    row.TAMACNIVEL = this.utilities.calcTamacNivel(row.NIVEL, 'produto');
    row.CDNVPRODUTO = row.NIVEL ? this.utilities.getReportMaskNivel(mask, row.NIVEL) : 0;
    row.REPORT_NAME = row.REPORT.REPORT_NAME;
    row.CD_REPORT_CODE = row.REPORT.CD_REPORT_CODE;
  }
  
  public generateReport({ component }: IEventParam<Button>) {
    const form = Metadata.getInstance<Form>('formRelatorioNutriProd');
    const row: IDictionary = form.value;
    
    row.reportFormat = component.name;
    this.setParams(row);
    
    this.utilities.activeLoadingButton(component);
    
    this.reportController.generateReport('report/NutriProd', row, row.CD_REPORT_CODE).finally(() => {
      this.utilities.disableLoadingButton(component);
    });
  }
  
  public setfullscreen({ component }: IEventParam<Button>) {
    this.modalAddNutri.fullscreen = !this.modalAddNutri.fullscreen;
    this.modalAddNutri.cssClass = this.modalAddNutri.fullscreen ? 'fullscreen-fill-content' : 'modal-emissao-cardapio';
    
    component.iconName = (this.modalAddNutri.fullscreen) ? 'mdi-fullscreen-exit' : 'mdi-fullscreen';
  }
}
